#include <SoftwareSerial.h>

#define TxPin 13      // Tx pin 13, connect to sensor Rx pin 5
#define RxPin 12      // Rx pin 12, connect to sensor Tx pin 7
#define BUFFSIZE 15   // used for both sensor & PC buffers

//==============================================================================
SoftwareSerial GSSSerial(RxPin,TxPin); //Rx Tx
// RdPCBuffer is a linaer buffer, read full command line before sending to sensor
// RdBuffer is a circular buffer, echoing characters from sensor to PC
char RdBuffer[BUFFSIZE], RdPCBuffer[BUFFSIZE];
int InS,OutS,PCPtr;   // pointers used to control the buffers

//==============================================================================
void setup()
{
  pinMode(RxPin, INPUT);            // Sesnor Rx
  pinMode(TxPin, OUTPUT);           // Sensor Tx
  InS=0;                            // initialise buffer pointers
  OutS=0;
  PCPtr=0;
  Serial.begin(115200);             // to USB, initialise serial comms via USB (baud rate can be changed)
  GSSSerial.begin(9600);            // sensor serial comms (change to 38400 for SprintIR-R & Blink)
//  GSSSerial.print("*\r\n");         // send debug command if required, to confirm serial comms
}

//==============================================================================
void loop()
{
  if(ReadSensor())                  // check for data from sensor
    {
    do
      {
      Serial.print(RdBuffer[OutS]); // data available, send to terminal (PC)
      if(++OutS==BUFFSIZE)          // circular buffer, wrap ptr back to start
        OutS=0;
      }while(OutS!=InS);            // output all available chars
    }
  if(ReadPC())                      // only true when complete command received ending with'\n'
    {
    RdPCBuffer[PCPtr]=0;            // null terminate string
    GSSSerial.print(RdPCBuffer);    // send string received from PC to Sensor
    PCPtr=0;                        // reset string
    }
}

//==============================================================================
int ReadSensor(void)
{
if(GSSSerial.available())           // have any chars been received from sensor
  {
  RdBuffer[InS++] = GSSSerial.read(); // read char
  if(InS==BUFFSIZE)                 // if end of allocated buffer memory
    InS=0;                          // reset pointer, to create a circular buffer
  return true;                      // char received
  }
return false;                       // no char received
}

//==============================================================================
int ReadPC(void)
{
if(Serial.available())              // has any char been received from PC
  {
  RdPCBuffer[PCPtr] = Serial.read(); // Yes, copy char to buffer 
  if(RdPCBuffer[PCPtr++]=='\n')     // was it new line character
    return true;                    // full line, return true
  }
return false;                       // \n not yet received
}

//==============================================================================
